"""Tests for schemadrift.

Note: Plugin-specific tests are located within each plugin directory for self-containment:
  - src/schemadrift/plugins/builtin/view/tests.py
  - src/schemadrift/plugins/builtin/schema/tests.py
  - src/schemadrift/plugins/builtin/database/tests.py

Run all tests with: pytest src/ tests/ -v
"""
